#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "config.h"
#include "global.h"
#include "buffers.h"

struct buf_s {
	int		n;
	char		*buf;
	int		size;
	struct buf_s	*next;
};
typedef struct buf_s Buf;


#define NULL_buf		(Buf *)0

static Buf	*Cur		= NULL_buf;
static int	Num		= 0;
static char	tmp[20];

static Buf	*new();
static int	replace();
static void	free_buf();


/* add query to the buffer */

int
buf_next(qbuf)
char *qbuf;
{
	Cur = Cur->next;
	Cur->n = ++Num;
	return(replace(Cur,qbuf));
}

char *
buf_get(num)
register int num;
{
	register Buf *bp = Cur;
	register int i;


	if(num < 0){
		sprintf(tmp,"- %d",num);
		Err(ErrUnosuchbuf,tmp);
		return(NULL_char);
	}

	if(num == 0)
		return(bp->buf);

	for(i = 0; i < def_buffers; i++){
		if(bp->n == num)
			return(bp->buf);
		bp = bp->next;
	}
	sprintf(tmp,"- %d",num);
	Err(ErrUnosuchbuf,tmp);
	return(NULL_char);
}

int
buf_init()
{
	int	i,begin;
	Buf	*bp, *first, *last;

	first = NULL_buf;
	begin = 1;
	for(i = 0; i < def_buffers; i++){
		if((bp = new()) == NULL_buf)
			return(RET_MEM_ERR);

		if(begin){		/*save first created buffer*/
			last = bp;
			begin = 0;
		}

		bp->next = first;
		first = bp;
	}

	/*make ring*/
	last->next = first;
	Cur = first;

	return(RET_OK);
}

int
buf_run_append(qbuf)
char	*qbuf;
{
	register char	*cp, *new, *obuf;
	register int	i;
	char		*cmd;
	int		ret;

	cp	= qbuf;
	new	= "\0";

	SKIP_WORD(cp); *cp++ = '\0';
	SKIP_SPACES(cp);	/* cp -> appended text */
	new = cp;
	obuf = buf_get(atoi(qbuf));

	if(obuf == NULL_char)
		return(RET_ERR);

	i = 0;

	if(*new != '\0'){		/* append new text */
		cmd = Qbuf;
		cp = cmd;
		i = 1;
		while(i++ < QBUF_SIZE){
			*cp++ = *obuf++;
			if(*obuf == '\0')
				break;
		}
		*cp++ = ' '; i++;
		if(i >= QBUF_SIZE){
			Err(ErrUtoolong,NULL_char);
			return(RET_ERR);
		}
		while(i++ < QBUF_SIZE){
			*cp++ = *new++;
			if(*new == '\0')
				break;
		}
		if(i > QBUF_SIZE){
			Err(ErrUtoolong,NULL_char);
			return(RET_ERR);
		}
		*cp = '\0';
	} else
		cmd = obuf;

	Out("\n");Out(cmd);Out("\n");

	ret = doquery(cmd);

	if(ret == RET_OK || ret == RET_ERR){
		if(i) /* new command */
			return(buf_next(cmd));
		return(RET_OK);
	}
	return(ret);
}

/* commands ******************************************************************/

int
buf_num_cmd(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	int	i, num, begin;
	Buf	*bp, *first, *last;

	if((num = atoi(argv[1])) <= def_buffers){
		sprintf(tmp,"Current - %d",def_buffers);
		Err(ErrUnumbufsmall,tmp);
		return(RET_ERR);
	}

	if(num > 999){
		Err(ErrUnumbufbig,"Max - 999");
		return(RET_ERR);
	}

	num = num - def_buffers;

	first = NULL_buf;

	begin = 1;
	for(i = 0; i < num; i++){
		if((bp = new()) == NULL_buf){
			free_buf(first);
			return(RET_ERR);
		}

		if(begin){		/*save first created buffer*/
			last = bp;
			begin = 0;
		}

		bp->next = first;
		first = bp;
	}

	/*insert to the ring*/

	bp = Cur->next;		/* save old buffer */
	Cur->next = first;	/* insert */
	last->next = bp;	/* restore old*/

	def_buffers += num;

	return(RET_OK);
}

int
buf_edit_cmd(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	char	*cp,*new;
	int	ret;

	if(argc < 2){
		cp = Cur->buf;
		if(cp == NULL_char)
			cp = "";
	} else {
		if((cp = buf_get(atoi(argv[1]))) == NULL_char)
			return(RET_ERR);
	}

	if((new = edit(cp)) == NULL_char)
		return(RET_OK);

	if((ret = doquery(new)) > -1)
		return(buf_next(new));

	return(ret);
}

int
buf_run_cmd(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	char	*cp;

	if(argc < 2)
		cp = Cur->buf;
	else
		cp = buf_get(atoi(argv[1]));

	if(cp == NULL_char)
		return(RET_ERR);

	Out("\n");Out(cp);Out(";\n");

	return(doquery(cp));
}

int
buf_list(bnum)
int bnum;
{
	int		n,i;
	Buf		*bp;
	register char	*cp;

	bp = Cur->next;

	if(Num == 0)
		return(RET_ERR);

	if(bnum > 0){
		if(bnum > def_buffers)
			n = def_buffers;
		else {
			n = bnum;
			i = def_buffers - bnum;
			while(i-- > 0) bp = bp->next;
		}
	} else
		n = def_buffers;

	for(i = 0; i < n; i++){
		if(bp->buf != NULL_char){
			sprintf(tmp,"\n[%3d] ",bp->n);
			Out(tmp);
			cp = bp->buf;
			while(*cp != '\0'){
				if(*cp == '\n')
					Out("\n      ");
				else
					fputc(*cp,stdout);
				cp++;
			}
			Out("\n");
		}
		bp = bp->next;
	}

	Out("\n");
	return(RET_OK);
}

int
buf_save(num,file,opt)
int num;
char *file;
char *opt;
{
	char *bp;

	if(num < 1 && Cur->buf == NULL_char){
		Err(ErrUnosuchbuf,"(last buffer)");
		return(RET_ERR);
	}

	if((bp = buf_get(num)) == NULL_char)
		return(RET_ERR);

	return(save(bp,file,opt));
}


/***************************************************************************/


static int
replace(buf,str)
Buf *buf;
char *str;
{
	int	size;

	size = strlen(str) + 1;

	if(buf->size < size){
		free_char(buf->buf);
		if((buf->buf = (char *)malloc(size)) == NULL){
			Err(ErrIalloc,"(Buf replace)");
			return(RET_MEM_ERR);
		}
		buf->size = size;
	}
	strcpy(buf->buf,str);
	return(RET_OK);
}

static Buf *
new()
{
	Buf *bp;

	if((bp = (Buf *)calloc(1,sizeof(Buf))) == NULL){
		Err(ErrIalloc,"(Buf new)");
		return(NULL_buf);
	}
	bp->buf = NULL_char;
	bp->size = 0;
	bp->n = 0;
	bp->next = NULL_buf;
	return(bp);
}

static void
free_buf(buf)
Buf *buf;
{
	register Buf *bp, *bp1;

	bp = buf;
	while(bp != NULL_buf){
		bp1 = bp->next;
		free_char(bp->buf);
		free((FREETYPE *)bp);
		bp = bp1;
	}
}

