#include <stdio.h>
#include <string.h>
#include "config.h"
#include "compat.h"
#include "global.h"
#include "column.h"
#include "commands.h"

static int	do_display();
static int	do_size();
static int	do_title();
static int	do_just();
static int	do_format();
static int	do_wrap();
static int	do_like();

static Command ColumnCmds[] = {
	"form",		2,	do_format,
	"jus",		2,	do_just,
	"lik",		2,	do_like,
	"on",		0,	do_display,
	"off",		0,	do_display,
	"siz",		2,	do_size,
	"tit",		2,	do_title,
	"wra",		2,	do_wrap,
	NULL_char,	0,	NULL
};

static void	column_show();
static int	save_column();

static char tmp[2048];

int
column_cmd(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	char	*colname, *tname;
	int	ret, new = 0;
	ColF	*cfp,*cfp1,*cfp2;


	if(argc < 2){
		column_show(NULL_ColF,0);
		return(RET_OK);
	}

	tname = argv[1];

	if((colname = strchr(tname,'.')) == NULL){
		colname = tname;
		tname = "*";
	} else
		*colname++ = '\0';

	cfp = colf_get(tname,colname,1);

	if(argc == 2){
		if(cfp == NULL_ColF){
			Err(ErrUnosuchcol,NULL_char);
			return(RET_ERR);
		}
		column_show(cfp,1);
		return(RET_OK);
	}

	if(strncasecmp(argv[2],"cle",3) == 0){ /* CLEAR */
		if(cfp == NULL_ColF){
			Err(ErrUnosuchcol,NULL_char);
			return(RET_ERR);
		}
		if(cfp == Columns){
			cfp = Columns->next;
			colf_free(Columns);
			Columns = cfp;
			return(RET_OK);
		}
		cfp1 = cfp2 = Columns;
		while(cfp1 != NULL_ColF){
			if(cfp == cfp1){
				cfp2->next = cfp1->next;
				colf_free(cfp1);
				break;
			}
			cfp2 = cfp1;
			cfp1 = cfp1->next;
		}
		return(RET_OK);
	}


	if(cfp == NULL_ColF){
		new = 1;
		if((cfp = colf_new(tname,colname)) == NULL_ColF)
			return(RET_MEM_ERR);
	}

	/* skip command name and column name */
	argc--; argv++;

	ret = subcmd(ColumnCmds,argc,argv,(void *)cfp);

	switch(ret){
	case RET_PARSER_ERR:
	case RET_ERR:
		if(new)
			colf_free(cfp);
		return(RET_OK);
		break;
	case RET_OK:
		if(new){
			/* put on top */
			cfp->next = Columns;
			Columns = cfp;
		}
	}
	return(ret);
}

static int
do_like(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	ColF	*from,*to = (ColF *)p;
	char	*colname, *tname;

	tname = argv[1];

	if((colname = strchr(tname,'.')) == NULL){
		colname = tname;
		tname = "*";
	} else
		*colname++ = '\0';

	if((from = colf_get(tname,colname,1)) == NULL_ColF){
		Err(ErrUnosuchcol,NULL_char);
		return(RET_ERR);
	}

	if(from->tsize)
		strcpy(to->title,from->title);

	to->display = from->display;
	to->size = from->size;
	to->just = from->just;
	to->tsize = from->tsize;
	to->tjust = from->tjust;
	to->wrap = from->wrap;
	free_char(to->numf);
	to->numf = copy_str(from->numf);

	return(RET_OK);
}

static int
do_display(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	ColF	*cfp = (ColF *)p;

	return(boolcmd(argv[0],&cfp->display));
}

static int
do_size(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	ColF	*cfp = (ColF *)p;
	int	size;

	if((size = atoi(argv[1])) < 1){
		Err(ErrUcol_size,NULL_char);
		return(RET_ERR);
	}

	cfp->size = size;

	return(RET_OK);
}

static int
do_title(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	ColF	*cfp = (ColF *)p;
	int	size;

	if((size = strlen(argv[1])) > MAX_CTIT_LEN){
		Err(ErrUcol_tsize,NULL_char);
		return(RET_ERR);
	}
	cfp->tsize = size;
	strcpy(cfp->title,argv[1]);

	return(RET_OK);
}

static int
do_just(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	ColF	*cfp = (ColF *)p;

	switch(argv[1][0]){
	case 'r':
	case 'R':
		cfp->just = J_RIGHT;
		break;
	case 'l':
	case 'L':
		cfp->just = J_LEFT;
		break;
	case 'c':
	case 'C':
		cfp->just = J_CENTR;
		break;
	case 't':
	case 'T':
		if(argc < 3){
			Err(ErrUsynargs,NULL_char);
			return(RET_ERR);
		}
		switch(argv[2][0]){
		case 'r':
		case 'R':
			cfp->tjust = J_RIGHT;
			break;
		case 'l':
		case 'L':
			cfp->tjust = J_LEFT;
			break;
		case 'c':
		case 'C':
			cfp->tjust = J_CENTR;
			break;
		default:
			Err(ErrUcol_just,NULL_char);
			return(RET_ERR);
		}
		break;
	default:
		Err(ErrUcol_just,NULL_char);
		return(RET_ERR);
	}
	return(RET_OK);
}


static int
do_format(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	ColF	*cfp = (ColF *)p;
	char	*f;

	/* check syntax */
	if((f = makenumformat(argv[1],NUMF_INT)) == NULL_char)
		return(RET_ERR);
	free_char(f);

	/* check syntax */
	if((f = makenumformat(argv[1],NUMF_FLOAT)) == NULL_char)
		return(RET_ERR);
	free_char(f);

	free_char(cfp->numf);

	if((cfp->numf = (char *)malloc(strlen(argv[1])+1)) == NULL){
		Err(ErrIalloc,NULL_char);
		return(RET_MEM_ERR);
	}

	strcpy(cfp->numf,argv[1]);

	return(RET_OK);

}

static int
do_wrap(argc,argv,p)
int	argc;
char	*argv[];
void	*p;
{
	ColF	*cfp = (ColF *)p;

	switch(argv[1][0]){
	case 'l':
	case 'L':
		cfp->wrap = WRAP_LINE;
		break;
	case 's':
	case 'S':
		cfp->wrap = WRAP_SIZE;
		break;
	case 't':
	case 'T':
		cfp->wrap = WRAP_TRUNC;
		break;
	case 'w':
	case 'W':
		cfp->wrap = WRAP_WORD;
		break;
	default:
		Err(ErrUsynwrap,NULL_char);
		return(RET_ERR);
	}
	return(RET_OK);
}

/**************************************************************************/

int
column_list(col)
char *col;
{
	register ColF	*cfp;
	register char *cp, *cname, *tname;
	int anytab, anycol;

	if(Columns == NULL_ColF)
		return(RET_OK);

	if(col == NULL_char){
		column_show(NULL_ColF,0);
		return(RET_OK);
	}


	if((cp = strchr(col,'.')) != NULL){
		*cp++ = '\0';
		tname = col;
		cname = cp;
	} else {
		tname = "\0";
		cname = col;
	}

	anytab = anycol = 0;

	if(*tname == '\0' || *tname == '*')
			anytab = 1;

	if(*cname == '\0' || *cname == '*')
			anycol = 1;

	if(anytab && anycol){
		column_show(NULL_ColF,0);
		return(RET_OK);
	}

	cfp = Columns;

	while(cfp != NULL_ColF){
		if(!anytab && strcmp(tname,cfp->table)){
				cfp = cfp->next;
				continue;
		}
		if(!anycol && strcmp(cname,cfp->name)){
				cfp = cfp->next;
				continue;
		}
		column_show(cfp,1);
		cfp = cfp->next;
	}

	return(RET_OK);
}

int
column_save(col,fname,opt)
char *col;
char *fname;
char *opt;
{
	register ColF	*cfp;
	register char *cp, *cname, *tname, *o;
	int anytab, anycol;

	if(Columns == NULL_ColF)
		return(RET_OK);

	if(col == NULL_char)
		return(save_column(NULL_ColF,fname,opt,0));


	if((cp = strchr(col,'.')) != NULL){
		*cp++ = '\0';
		tname = col;
		cname = cp;
	} else {
		tname = "\0";
		cname = col;
	}

	anytab = anycol = 0;

	if(*tname == '\0' || *tname == '*')
			anytab = 1;

	if(*cname == '\0' || *cname == '*')
			anycol = 1;

	if(anytab && anycol)
		return(save_column(NULL_ColF,fname,opt,0));

	cfp = Columns;

	o = opt;
	while(cfp != NULL_ColF){
		if(!anytab && strcmp(tname,cfp->table)){
				cfp = cfp->next;
				continue;
		}
		if(!anycol && strcmp(cname,cfp->name)){
				cfp = cfp->next;
				continue;
		}
		if(save_column(cfp,fname,o,1) != RET_OK)
			return(RET_ERR);
		o = "append";
		cfp = cfp->next;
	}

	return(RET_OK);
}
/**************************************************************************/

#define NORM(S,V)	sprintf(tmp,S,V);Out(tmp)
#define BOOL(S,V)	sprintf(tmp,S, V ? "ON" : "OFF");Out(tmp)

static void
column_show(cfp,one)
register ColF	*cfp;
int one;
{
	if(cfp == NULL_ColF){
		cfp = Columns;
		one = 0;
	}

     while(cfp != NULL_ColF){
	Out("\n");
	 sprintf(tmp,"%s.%s\n",cfp->table,cfp->name);
	Out(tmp);
		BOOL("            Display:  %s\n",cfp->display);
	if(cfp->size)
	 sprintf(tmp,"            Size:     %d\n",cfp->size);
	else
	 sprintf(tmp,"            Size:     (default)\n");
	Out(tmp);
	 sprintf(tmp,"            Justify:  data - %s, title - %s\n",
			JustType[cfp->just],JustType[cfp->tjust]);
	Out(tmp);
	if(cfp->tsize)
	 sprintf(tmp,"            Title:    '%s'\n",cfp->title);
	else
	 sprintf(tmp,"            Title:    (column name)\n");
	Out(tmp);
	if(cfp->numf != NULL_char)
	 sprintf(tmp,"            Format:   '%s'\n",cfp->numf);
	else
	 sprintf(tmp,"            Format:   (default)\n");
	Out(tmp);
		NORM("            Wrap:     %s\n",WrapType[cfp->wrap]);

	if(one)
		break;

	cfp = cfp->next;
     }
}

static int
save_column(cfp,fname,opt,one)
ColF	*cfp;
char *fname;
char *opt;
int one;
{
	char col[128],num[32], *o;
	int p;

	if(cfp == NULL_ColF){
		cfp = Columns;
		one = 0;
	}

	o = opt;

     	while(cfp != NULL_ColF){
		tmp[0] = '\0';
		if(cfp->table[0] == '*')
			sprintf(col,"COLUMN %s ",cfp->name);
		else
			sprintf(col,"COLUMN %s.%s ",cfp->table,cfp->name);

		p = def_pause;
		def_pause = FALSE;
		Out("Saving "); Out(col); Out("...\n");
		def_pause = p;

		if(!cfp->display){
			strcpy(tmp,col);
			strcat(tmp,"OFF");
		}
		if(cfp->size){
			if(tmp[0] != '\0')
				strcat(tmp,";\n");
			strcat(tmp,col);
			sprintf(num,"SIZE %d",cfp->size);
			strcat(tmp,num);
		}
		if(cfp->just != J_NONE){
			if(tmp[0] != '\0')
				strcat(tmp,";\n");
			strcat(tmp,col);
			strcat(tmp,"JUSTIFY ");
			strcat(tmp,JustType[cfp->just]);
		}
		if(cfp->title[0] != '\0'){
			if(tmp[0] != '\0')
				strcat(tmp,";\n");
			strcat(tmp,col);
			strcat(tmp,"TITLE '");
			strcat(tmp,cfp->title);
			strcat(tmp,"'");
		}
		if(cfp->tjust != J_NONE){
			if(tmp[0] != '\0')
				strcat(tmp,";\n");
			strcat(tmp,col);
			strcat(tmp,"JUSTIFY TITLE ");
			strcat(tmp,JustType[cfp->tjust]);
		}
		if(cfp->numf != NULL_char){
			if(tmp[0] != '\0')
				strcat(tmp,";\n");
			strcat(tmp,col);
			strcat(tmp,"FORMAT '");
			strcat(tmp,cfp->numf);
			strcat(tmp,"'");
		}
		if(cfp->wrap != WRAP_NONE){
			if(tmp[0] != '\0')
				strcat(tmp,";\n");
			strcat(tmp,col);
			strcat(tmp,"WRAP ");
			strcat(tmp,WrapType[cfp->wrap]);
		}
		if(tmp[0] != '\0'){
			if(save(tmp,fname,o) != RET_OK)
				return(RET_ERR);
		}
		if(one)
			break;
		o = "append";
		cfp = cfp->next;
	}
	return(RET_OK);
}

