#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "config.h"
#include "global.h"
#include "commands.h"
#include "buffers.h"

static int	exit_cmd();
static int	start_cmd();
static int	spool_cmd();
static int	help_cmd();
static int	view_cmd();
static int	list_cmd();
static int	save_cmd();
static int	shell_cmd();

static Command Commands[] = {
	"!",		0,	shell_cmd,
	"col",		0,	column_cmd,	/*sql+ column*/
	"con",		2,	connect_cmd,	/*sql+ connect */
	"disc",		0,	disconnect_cmd, /*sql+ disconnect*/
	"exit",		0,	exit_cmd,	/*sql+ exit*/
	"e",		0,	buf_edit_cmd,
	"hel",		0,	help_cmd,
	"l",		0,	list_cmd,
	"quit",		0,	exit_cmd,
	"r",		0,	buf_run_cmd,	/*sql+ run*/
	"sav",		2,	save_cmd,
	"set",		0,	set_cmd,	/*sql+ set*/
	"spo",		2,	spool_cmd,	/*sql+ spool*/
	"s",		2,	start_cmd,	/*sql+ start*/
	"v",		2,	view_cmd,
	NULL_char,	0,	NULL,
};

/* Implemented SQL commands */
static char *sqlCommands[] = {
	"create",
	"delete",
	"drop",
	"insert",
	"select",
	"update",
	NULL_char
};


int
scanner(buf)
char	*buf;
{
	register char 	*bp,**cp;
	int		ret, sql = 0;

	if (buf == NULL || *buf == '\0'){
		Err(ErrUemptyq,NULL_char);
		return(RET_ERR);
	}

	bp = buf; SKIP_SPACES(bp);

	if(isdigit(*bp)) /* rerun command from SQL buffer */
		return(buf_run_append(bp));

	for(cp = sqlCommands; *cp != NULL_char; cp++){
		if(strncasecmp(bp,*cp,strlen(*cp)) == 0){
			sql = 1;
			break;
		}
	}

	if(sql){
		if((ret = doquery(bp)) < 0)
			return(ret);
		return(buf_next(bp));
	}

	return(cmdparse(Commands,bp,NULL));	
}

int
cmdparse(cmds,buf,p)
Command		cmds[];
char		*buf;
void		*p;
{
	register char	*cp, *line;
	Command		*cmdp;
	char		*argv[NUM_ARGS];
	int		argc, qflag;

	line = buf;

	for(argc = 0; argc < NUM_ARGS; argc++)
		argv[argc] = NULL_char;

	argc = 0;
	while(argc < NUM_ARGS){
		qflag = 0;

		SKIP_SPACES(line);

		if(*line == '\0')
			break;

		/* Check for quoted token */
		if(*line == '\''){
			line++;	/* Suppress quote */
			qflag = 1;
		}

		argv[argc++] = line;

		/* Find terminating delimiter */
		if(qflag){
			/* Find quote, it must be present */
			if((line = strchr(line,'\'')) == NULL){
				Err(ErrUsynQuote,NULL_char);
				return(RET_PARSER_ERR);
			}
			*line++ = '\0';
		} else {
			cp = line;
			SKIP_WORD(cp);
			if(*cp != '\0')
				*cp++ = '\0';
			else
				break;

			/* Check for next word */
			SKIP_SPACES(cp);
			if(*cp == '\0')
				break;  /* it was last word */

			line = cp;
		}
	}

	if (argc < 1) {		/* empty command line */
		argc = 1;
		argv[0] = "";
	}

	/* Lines beginning with "#" are comments */
	if(argv[0] == NULL_char || argv[0][0] == '#')
		return(RET_OK);

	/* Look up command in table*/
	for(cmdp = cmds; cmdp->name != NULL_char; cmdp++){
		if(strncasecmp(argv[0],cmdp->name,strlen(cmdp->name)) == 0)
			break;
	}

	if(cmdp->name == NULL_char) {
		Err(ErrUunknowncmd,NULL_char);
		return(RET_PARSER_ERR);
	} else {
		if(argc < cmdp->min) {
			/* Insufficient arguments */
			Err(ErrUsynargs,NULL_char);
			return(RET_PARSER_ERR);
		}
	}

	return((*cmdp->cmd)(argc,argv,p));
}


int
subcmd(tab,argc,argv,p)
Command		tab[];
int		argc;
char		*argv[];
void		*p;
{
	register Command *cmdp;
	char		**pargv;
	int		found = 0;

	if(argc < 2){
		Err(ErrIargc,"subcmd()");
		return(RET_INTERNAL_ERR);
	}
	/* skip command name */
	argc--; argv++;

	for(cmdp = tab; cmdp->name != NULL_char; cmdp++){
		if(strncasecmp(argv[0],cmdp->name,strlen(cmdp->name)) == 0){
			found = 1;
			break;
		}
	}

	if(!found){
		Err(ErrUunknownscmd,NULL_char);
		return(RET_PARSER_ERR);
	}

	if(argc < cmdp->min){
		Err(ErrUsynargs,NULL_char);
		return(RET_PARSER_ERR);
	}

	return((*cmdp->cmd)(argc,argv,p));
}

int
boolcmd(str,i)
char	*str;
int	*i;
{

	if(strcasecmp(str,"on") == 0)
		*i = TRUE;
	else if(strcasecmp(str,"yes") == 0)
		*i = TRUE;
	else if(strcasecmp(str,"off") == 0)
		*i = FALSE;
	else if(strcasecmp(str,"no") == 0)
		*i = FALSE;
	else
		return(RET_ERR);

	return(RET_OK);
}


static int
exit_cmd(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	Out("\nBye.\n\n");
	return(RET_EXIT);
}

static int
spool_cmd(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	char *file,tmpfile[128];

	if(strcasecmp(argv[1],"off") == 0){
		SpoolClose();
		return(RET_OK);
	}
	if(strcasecmp(argv[1],"out") == 0){
		if(def_spoolfile == NULL_char){
			Err(ErrUnospool,NULL_char);
			return(RET_ERR);
		}
		strcpy(tmpfile,def_spoolfile);
		SpoolClose();
		return(outfile(tmpfile,TRUE));
	}
	file = makefilename(argv[1],def_suf_lst);
	return(SpoolOpen(file));
}

static int
start_cmd(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	char	*file;

	file = makefilename(argv[1],def_suf_cmd);
	file = prependdir(file,def_cmddir);

	if(InSet(file) == RET_ERR){
		Err(ErrUstart_file,"E");
		return(RET_ERR);
	}

	return(RET_OK);
}

static int
help_cmd(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	char tmp[128], *dp;


	dp = HELP_DIR;

	if(argc > 3){
		sprintf(tmp,"%s/%s.%s.%s.hlp",dp,argv[1],argv[2],argv[3]);
		if(outfile(tmp,FALSE) == RET_OK)
			return(RET_OK);
	}

	if(argc > 2){
		sprintf(tmp,"%s/%s.%s.hlp",dp,argv[1],argv[2]);
		if(outfile(tmp,FALSE) == RET_OK)
			return(RET_OK);
	}

	if(argc > 1){
		sprintf(tmp,"%s/%s.hlp",dp,argv[1]);
		if(outfile(tmp,FALSE) == RET_OK)
			return(RET_OK);
	}

	sprintf(tmp,"%s/general.hlp",dp);
	if(outfile(tmp,FALSE) == RET_OK)
		return(RET_OK);

	Err(ErrUnohelp,NULL_char);
	return(RET_ERR);
}

static int
view_cmd(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return(outfile(argv[1],TRUE));
}

static int
list_cmd(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	if(argc < 2)
		return(buf_list(0));

	if(isdigit(argv[1][0]))
		return(buf_list(atoi(argv[1])));

	if(strncasecmp(argv[1],"col",3) != 0){
		Err(ErrUsyntax,NULL_char);
		return(RET_ERR);
	}

	if(argc < 3)
		return(column_list(NULL_char));

	return(column_list(argv[2]));
}

static int
save_cmd(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	char *file,*opt;

	file = makefilename(argv[1],def_suf_cmd);
	file = prependdir(file,def_cmddir);

	if(argc < 3)
		return(buf_save(0,file,NULL_char));

	if(isdigit(argv[2][0])){
		if(argc < 4)
			opt = NULL_char;
		else
			opt = argv[3];

		return(buf_save(atoi(argv[2]), file, opt));
	}

	if(strncasecmp(argv[2],"col",3) != 0){
		Err(ErrUsyntax,NULL_char);
		return(RET_ERR);
	}

	if(argc < 4)
		return(column_save(NULL_char,file,NULL_char));

	if(argc < 5)
		opt = NULL_char;
	else
		opt = argv[4];

	return(column_save(argv[3],file,opt));
}


static int
shell_cmd(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	char tmp[256];
	int i;

	if(argc < 2){
		if(system(def_shell) < 0){
			Err(ErrUshellexec,"E");
			return(RET_ERR);
		}
		Out("\n");
		return(RET_OK);
	}

	tmp[0] = '\0';

	for(i = 1; i < argc; i++){
		strcat(tmp,argv[i]);
		strcat(tmp," ");
	}

	if(system(tmp) < 0){
		Err(ErrUshellexec,"E");
		return(RET_ERR);
	}

	return(RET_OK);
}
