#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include "config.h"
#include "compat.h"
#include "global.h"

#define IBUF_SIZE	1024			/* length of input string */

#define OFLAGS_CREATE	(O_WRONLY | O_CREAT | O_EXCL)
#define OFLAGS_REPLACE	(O_WRONLY | O_CREAT | O_TRUNC)
#define OFLAGS_APPEND	(O_WRONLY | O_CREAT | O_APPEND)

#define DEF_SAVE_FLAG	OFLAGS_REPLACE
#define DEF_SAVE_MODE	0644

#define NULL_file	(FILE *)NULL

static char	Ibuf[IBUF_SIZE+2];
static char	tmp[IBUF_SIZE+2];

static FILE	*Ifile		= stdin;
static FILE	*Efile		= stdout;
static FILE	*Sfile		= NULL_file;	/* spool file */
static int	I_from_file	= FALSE;
static char	*CurPrompt	= NULL_char;
static int	Silent		= FALSE;
static int	Line		= 1;		/* screen cursor for messages*/
static int	More		= TRUE;

static int	Fline		= 1;		/* spool file cursor */
static int	EndBar		= FALSE;
static int	PageNew		= TRUE;
static int	PageRows	= FALSE;
static int	PageBegin	= TRUE;

static int	Sline		= 1;		/* screen cursor for reports */
static int	ScreenEndBar	= FALSE;
static int	ScreenNew	= TRUE;
static int	ScreenRows	= FALSE;
static int	ScreenBegin	= TRUE;

static int	get_oflag();

void
Out(str)
char *str;
{
	register char *cp;
	int	ss;

	if(Silent || !def_out_messag || !More)
		return;

	if(!def_pause){
		fputs(str,stdout);
		return;
	}

	cp = str;
	ss = def_scrsize - 2; /* 2 line for the prompt */

	while(*cp != '\0'){
		fputc(*cp,stdout);
		if(*cp == '\n'){
			Line++;
			if(Line > ss){
			  tmp[0] = '\0';
			  fputs("\nContinue [Y/n]: ",stdout);
			  (void)fgets(tmp,IBUF_SIZE,stdin);
			  fputc('\n',stdout);
			  if(*tmp == 'n' || *tmp == 'N'){
				More = FALSE;
				return;
			  }
			  Line = 1;
			}
		}
		cp++;
	}

}

void
OutSilent()
{
	Efile = stderr;
	Silent = TRUE;
	def_pause = FALSE;
	def_out_messag = FALSE;
}


void
ReportOut(rec,head,sep,end,rowsep,nullsep)
char *rec;
char *head;
char *sep;
char *end;
char *rowsep;
char *nullsep;
{
	int	i;


	if(Sfile != NULL_file){
		if(Fline > def_page_size || PageNew){ /* start new page */
			Fline = 1;
			PageNew = FALSE;
			EndBar = FALSE; /*XXX 4.0*/
			PageBegin = TRUE;

			for(i = 0; i < def_margin_top; i++) {
				/* top margin */
				fputc('\n',Sfile);
				Fline++;
			}
			if(def_colsep){ /* top line */
				fputs(end,Sfile);
				fputc('\n',Sfile);
				Fline++;
			}
			if(def_heading){ /* print heading? */
				fputs(head,Sfile);fputc('\n',Sfile);
				Fline++;
				if(sep != NULL_char)
					fputs(sep,Sfile);
				fputc('\n',Sfile);
				Fline++;
			}
		} /* new page */

		if(rowsep != NULL_char){
		    if(PageRows && !PageBegin){
			if(Fline == PageEnd){
				if(nullsep != NULL_char)
					fputs(nullsep,Sfile);
				fputc('\n',Sfile);
				Fline++;
			} else {
				fputs(rowsep,Sfile);fputc('\n',Sfile);
				Fline++;
			}
		    }
		}
		if(rec != NULL_char){
			/* print record */
			fputs(rec,Sfile);fputc('\n',Sfile);
			Fline++;
			PageRows = TRUE;
			PageBegin = FALSE;

		}

		if(Fline > PageEnd){ /* end of page ?*/
			if(def_endbar){ /* print endbar ?*/
				if(end != NULL_char)
					fputs(end,Sfile);
				fputc('\n',Sfile);
				Fline++;
			}
			EndBar = TRUE;
			PageNew = TRUE; /*4.0 XXX*/
			if(def_page_break){
			   if(def_page_break == -1){
				for(i = 0; i < def_margin_bot; i++) {
					/* bot margin */
					fputc('\n',Sfile);
					/* Fline++; XXX 4.0*/
				}
			   } else {
				fputc(def_page_break,Sfile);
			   }
			} else { /*no page break */
			   fputc('\n',Sfile);
			}
		} /* end of page */
	} /* sfile */

	if(!def_out_report || !More)
		return;

	if(ScreenNew){ /* start new screen */
		Sline = 1;
		ScreenNew = FALSE;
		ScreenEndBar = FALSE;
		ScreenBegin = TRUE;
		fputc('\n',stdout); /* top margin */
		Sline++;
		if(def_colsep){ /* top line */
			fputs(end,stdout);
			fputc('\n',stdout);
			Sline++;
		}
		if(def_heading){ /* print heading? */
			fputs(head,stdout);fputc('\n',stdout);
			Sline++;
			if(sep != NULL_char)
				fputs(sep,stdout);
			fputc('\n',stdout);
			Sline++;
		}
	} /* new screen */

	if(rowsep != NULL_char){
		if(ScreenRows && !ScreenBegin){
			if(Sline == ScreenEnd){
				if(nullsep != NULL_char)
					fputs(nullsep,stdout);
				fputc('\n',stdout);
				Sline++;
			} else {
				fputs(rowsep,stdout);fputc('\n',stdout);
				Sline++;
			}
		}
	}

	if(rec != NULL_char){
		/* print row */
		fputs(rec,stdout);fputc('\n',stdout);
		Sline++;
		ScreenRows = TRUE;
		ScreenBegin = FALSE;
	}

	if(Sline > ScreenEnd){ /* end of screen ?*/
		if(def_endbar){ /* print endbar ?*/
			if(end != NULL_char)
				fputs(end,stdout);
			fputc('\n',stdout);
		}
		ScreenEndBar = TRUE;
		ScreenNew = TRUE;
		Sline = 1;
		fputc('\n',stdout);
		if(!def_pause){
			fputc('\n',stdout);
			return;
		}
		fputs("Continue [Y/n]: ",stdout);
		(void)fgets(tmp,IBUF_SIZE,stdin);
		if(*tmp == 'n' || *tmp == 'N'){
			More = FALSE;
			return;
		}
	} /* end of screen */
}


void
ReportEnd(end,msg)
char *end;
char *msg;
{
	int	i;


	if(Sfile != NULL_file){
		if(EndBar == FALSE){
			if(def_endbar && end != NULL_char){
				Fline++;
				fputs(end,Sfile);
				fputc('\n',Sfile);
			}
		}
		if(def_page_break){
			if(def_page_break == -1){
				for(i = Fline; i < def_page_size; i++)
					fputc('\n',Sfile);
			} else
				fputc(def_page_break,Sfile);
		} else
			fputc('\n',Sfile);
	}

	PageNew = TRUE;
	EndBar = FALSE;
	PageRows = FALSE;
	PageBegin = TRUE;
	Fline = 1;

	if(def_out_report && More) {
		if(ScreenEndBar == FALSE){
			if(def_endbar && end != NULL_char){
				Sline++;
				fputs(end,stdout);
				fputc('\n',stdout);
			}
			fputc('\n',stdout);Sline++;
		 }
		if(def_pause){
		   if(Sline > ScreenEnd){
			fputs("Press RETURN to continue...",stdout);
			(void)fgets(tmp,IBUF_SIZE,stdin);
			fputc('\n',stdout);
		   }
		}
	}
	if(msg != NULL_char){
		Out(msg);
	}

	ScreenNew = TRUE;
	ScreenEndBar = FALSE;
	ScreenRows = FALSE;
	ScreenBegin = TRUE;
	Sline = 1;
}


char *
Input()
{
	if(fgets(Ibuf,IBUF_SIZE,Ifile) == NULL)
		return(NULL_char);

	if(I_from_file && def_in_echo && !Silent){
		fputs(">> ",stdout);
		fputs(Ibuf,stdout);
	}

	return(Ibuf);
}

int
InSet(filename)
char	*filename;
{
	if(I_from_file == TRUE){
		fclose(Ifile);
	}

	if(filename == NULL_char){
		Ifile = stdin;
		I_from_file = FALSE;
		return(RET_OK);
	}

	if((Ifile = fopen(filename,"r")) == NULL){
		Ifile = stdin;
		I_from_file = FALSE;
		return(RET_ERR);
	}

	I_from_file = TRUE;
	return(RET_OK);
}

int
InFile()
{
	return(I_from_file);
}

void
Prompt(p)
char	*p;
{
	Line = 1;
	More = TRUE;

	if(I_from_file == TRUE)
		return;

	if(p != NULL_char)
		CurPrompt = p;

	if(Silent)
		return;

	fputs(CurPrompt,stdout);
}

void
oErr(msg)
char *msg;
{
	fputs(msg,Efile);
	fflush(Efile);
}

int
SpoolOpen(fname)
char *fname;
{
	FILE *fp;

	if((fp = fopen(fname,"w")) == NULL){
		Err(ErrIspoolwopen,"E");
		return(RET_ERR);
	}

	SpoolClose();

	if((def_spoolfile = (char *)malloc(strlen(fname)+1)) == NULL){
		Err(ErrIalloc,"(spool_open)");
		fclose(Sfile);
		unlink(fname);
		return(RET_MEM_ERR);
	}
	strcpy(def_spoolfile,fname);
	Sfile = fp;
	Out("Start spooling to: ");
	Out(def_spoolfile);
	Out("\n");
	return(RET_OK);
}

void
SpoolClose()
{
	if(Sfile == NULL_file)
		return;

	fclose(Sfile);
	Sfile = NULL_file;

	if(def_spoolfile != NULL_char){
		Out("Stop spooling to: ");
		Out(def_spoolfile);
		Out("\n");
		free((FREETYPE *)def_spoolfile);
	}
	def_spoolfile = NULL_char;
}


int
outfile(fname,err)
char	*fname;
int	err;
{
	FILE *fp;

	if(fname == NULL_char || *fname == '\0')
		return(RET_OK);

	if((fp = fopen(fname,"r")) == NULL){
		if(err == TRUE){
			sprintf(tmp,"Can't open file '%s' -",fname);
			Err(ErrIfopen,fname);
		}
		return(RET_ERR);
	}

	while(fgets(tmp,IBUF_SIZE,fp) != NULL)
		Out(tmp);

	fclose(fp);
	return(RET_OK);
}


char *
edit(buf)
char	*buf;
{
	register char *cp, *lastchar;
	int	fd;
	int	size;
	char	*ret = NULL_char;

	if((fd = open(DEF_TMPFILE,O_CREAT | O_WRONLY | O_TRUNC, 0777)) < 0){
		Err(ErrItmpfile,"E");
		return(NULL_char);
	}

	size = strlen(buf);

	if(write(fd,buf,size) < size){
		Err(ErrIfilewrite,"E");
		close(fd);
		goto edit_quit;
	}
	close(fd);
	sprintf(tmp,def_editor,DEF_TMPFILE);

	if(system(tmp) < 0){
		Err(ErrIcantedit,"E");
		goto edit_quit;
	}

	if((fd = open(DEF_TMPFILE,O_RDONLY, 0777)) < 0){
		Err(ErrItmpfile,"E");
		goto edit_quit;
	}

	bzero(Qbuf,QBUF_SIZE+2);

	if((size = read(fd,Qbuf,QBUF_SIZE)) < 0){
		Err(ErrItmpfile,"E");
		close(fd);
		goto edit_quit;
	}
	close(fd);

	if(size == 0)
		goto edit_quit;

	cp = Qbuf;

	while(*cp != '\0'){
		if(*cp != ' ' && *cp != '\t' && *cp != '\n' && *cp != '\r')
			lastchar = cp;
		cp++;
	}

	if(*lastchar == ';')	/* remove if present */
		*lastchar = '\0';

	ret = Qbuf;

edit_quit:
	unlink(DEF_TMPFILE);
	return(ret);
}
	

int
save(buf,fname,opt)
char	*buf;
char	*fname;
char	*opt;
{
	int		fd,flag;

	if(buf == NULL_char || *buf == '\0')
		return(RET_OK);

	if(fname == NULL_char || *fname == '\0'){
		Err(ErrUnofilename,"(save)");
		return(RET_ERR);
	}

	if(opt == NULL_char || *opt == '\0')
		flag = DEF_SAVE_FLAG;
	else {
		if((flag = get_oflag(opt)) == 0)
			return(RET_ERR);
	}


	if((fd = open(fname,flag,DEF_SAVE_MODE)) < 0){
		Err(ErrIsave_open,"E");
		return(RET_ERR);
	}

	if(flag == OFLAGS_APPEND)
		if(write(fd,"\n",1) < 1)
			goto save_quit;

	if(write(fd,buf,strlen(buf)) < 1)
		goto save_quit;
	if(write(fd,";\n",2) < 1)
		goto save_quit;

	close(fd);
	return(RET_OK);

save_quit:
	Err(ErrIfilewrite,"E");
	close(fd);
	unlink(fname);
	return(RET_ERR);
}

static int
get_oflag(str)
char *str;
{
	if(strncasecmp(str,"cre",3) == 0)
		return(OFLAGS_CREATE);
	if(strncasecmp(str,"rep",3) == 0)
		return(OFLAGS_REPLACE);
	if(strncasecmp(str,"app",3) == 0)
		return(OFLAGS_APPEND);

	Err(ErrUnosuchflag,NULL_char);
	return(0);
}


