#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "config.h"
#include "global.h"
#include "column.h"
#include "buffers.h"

#define EXIT_OK		0
#define EXIT_USAGE	1
#define EXIT_MEM	2
#define EXIT_INTERNAL	3

static char	Usage[]	= "\nUsage : emsql [-q] [-h host] [-d database]\n\n";
static char	Banner[] = "\n\
   SSql %s, %s.\n\
   Evgeny Mironov, FREEnet NOC, Moscow, Russia.\n\n";

static int	mainloop();


int
main(argc,argv)
int	argc;
char	*argv[];
{
	char	*cp;
	char	file[256];
	int	i,ret;

	strcpy(def_host,DEF_HOSTNAME);
	def_dbname[0] = '\0';

	for(i = 1;i<argc;i++){
	  if(argv[i][0] == '-'){
		switch(argv[i][1]){
		case 'h':			/*host*/
			if(++i >= argc){
				fputs(Usage,stderr);
				exit(EXIT_USAGE);
			}
			strncpy(def_host,argv[i],MAX_HOSTNAME);
			continue;
		case 'd':			/*dbname*/
			if(++i >= argc){
				fputs(Usage,stderr);
				exit(EXIT_USAGE);
			}
			strncpy(def_dbname,argv[i],MAX_NAME_LEN);
			continue;

		case 'q':			/* silent mode */
			OutSilent();
			continue;
		default:
			fputs(Usage,stderr);
			exit(EXIT_USAGE);
		}
	  }
	}

	sprintf(file,Banner,VERSION_NUM,VERSION_DATE); Out(file);


	/* Init global variables *********************************************/

	if(buf_init() != RET_OK)
		exit(EXIT_MEM);

	strcpy(def_fprompt,PROMPT_FIRST);
	strcpy(def_sprompt,PROMPT_SECOND);

	if((def_currency = (char *)malloc(strlen(DEF_CURRENCY) + 1)) == NULL){
		Err(ErrIalloc,"(init)");
		exit(EXIT_MEM);
	}
	strcpy(def_currency,DEF_CURRENCY);

	if((def_numformat = (char *)malloc(strlen(DEF_NUM_FORMAT) + 1)) == NULL){
		Err(ErrIalloc,"(init)");
		exit(EXIT_MEM);
	}
	strcpy(def_numformat,DEF_NUM_FORMAT);

	if((cp = getenv("VISUAL")) == NULL)
		cp = getenv("EDITOR");

	if(cp != NULL){
		if((def_editor = (char *)malloc(strlen(cp) + 4)) == NULL){
			Err(ErrIalloc,"(init)");
			exit(EXIT_MEM);
		}
		strcpy(def_editor,cp);
	} else {
		if((def_editor = (char *)malloc(strlen(DEF_EDITOR) + 4)) == NULL){
			Err(ErrIalloc,"(init)");
			exit(EXIT_MEM);
		}
		strcpy(def_editor,DEF_EDITOR);
	}
	if(strchr(def_editor,'%') == NULL)
		strcat(def_editor," %s");

	if((def_suf_cmd = (char *)malloc(strlen(DEF_SUFFIX_CMD) + 1)) == NULL){
		Err(ErrIalloc,"(init)");
		exit(EXIT_MEM);
	}
	strcpy(def_suf_cmd,DEF_SUFFIX_CMD);

	if((def_suf_lst = (char *)malloc(strlen(DEF_SUFFIX_LST) + 1)) == NULL){
		Err(ErrIalloc,"(init)");
		exit(EXIT_MEM);
	}
	strcpy(def_suf_lst,DEF_SUFFIX_LST);


	if((cp = getenv("SHELL")) != NULL){
		if((def_shell = (char *)malloc(strlen(cp) + 1)) == NULL){
			Err(ErrIalloc,"(init)");
			exit(EXIT_MEM);
		}
		strcpy(def_shell,cp);
	} else {
		if((def_shell = (char *)malloc(DEF_SHELL + 1)) == NULL){
			Err(ErrIalloc,"(init)");
			exit(EXIT_MEM);
		}
		strcpy(def_shell,DEF_SHELL);
	}

	/*********************************************************************/


	if((cp = getenv("HOME")) != NULL)
		sprintf(file,"%s/%s",cp,DEF_RCFILE);
	else
		strcpy(file,DEF_RCFILE);

	InSet(file); /* rcfile */
	ret = mainloop();

	SpoolClose();
	disconnect_db();

	switch(ret){
	case RET_OK:
	case RET_EXIT:
		exit(EXIT_OK);
	case RET_INTERNAL_ERR:
		exit(EXIT_INTERNAL);
	case RET_MEM_ERR:
		exit(EXIT_MEM);
	}
	exit(EXIT_INTERNAL);
}

static int
mainloop()
{
	register char *cp, *cp1, lastchar;
	register int len;
	char	qbuf[QBUF_SIZE+2];
	int	toolong,ret;

begin:
	(void)bzero(qbuf,QBUF_SIZE+2);
	cp = qbuf;
	toolong = 0;
	len = 0;
	ret = RET_OK;
	Prompt(def_fprompt);
	while((cp1 = Input()) != NULL_char){
		if(*cp1 == '#' || *cp1 == '\n' || *cp1 == '\0'){
			Prompt(NULL_char);
			continue;
		}

		while(*cp1 != '\n' && *cp1 != '\0'){
			if(++len >= QBUF_SIZE){
				toolong = 1;
				break;
			}
			lastchar = *cp1++;
			*cp++ = lastchar;
		}

		if(toolong){
			toolong = 0;
			Err(ErrUtoolong,NULL_char);
			cp = qbuf;
			(void)bzero(qbuf,QBUF_SIZE+2);
			len=0;
			Prompt(def_fprompt);
			continue;
		}

		if(lastchar == ';'){
			cp--; *cp = '\0';
			if((ret = scanner(qbuf)) < 0)
				return(ret);
			(void)bzero(qbuf,QBUF_SIZE);
			cp = qbuf;
			Prompt(def_fprompt);
			len=0;
		} else {
			*cp++ = '\n'; len++;
			Prompt(def_sprompt);
		}
	}

	if(InFile() == TRUE){
		InSet(NULL_char);
		goto begin;
	}
	return(RET_EXIT);
}


