#include <errno.h>
#include <string.h>
#include "config.h"
#include "global.h"

extern void oErr();

struct err_s {
	int	code;
	char	*msg;
};

static struct err_s Errors[] = {

/* USER errors *************************************************/

ErrUsyntax,		"Invalid syntax.",
ErrUsynargs,		"Invalid syntax - insufficient arguments.",
ErrUsynempty,		"Invalid syntax - parametr should not be empty.",
ErrUunknowncmd,		"Unknown command.",
ErrUunknownscmd,	"Invalid syntax - unknown subcommand.",
ErrUtoolong,		"Query string too long.",
ErrUemptyq,		"No query specified.",
ErrUnosuchcol,		"No such column.",
ErrUcol_size,		"Invalid column size.",
ErrUcol_tsize,		"Title lenght too long.",
ErrUcol_just,		"Invalid justification type.",
ErrUpage_s_small,	"Page size too small. Check size of margins. Current minimum ",
ErrUpage_w_small,	"Page width too small. Min - 5.",
ErrUpage_w_large,	"Page width too large.",

ErrUnumw_small,		"Numeric value width too small. Min - 1.",
ErrUnumw_large,		"Numeric value width too large.",
ErrUnumformat,		"Invalid format syntax at position -",

ErrUprompt_large,	"Prompt length too large.",
ErrUouttoolarge,	"Output line too large for the current page width.",
ErrUsynQuote,		"Invalid syntax - no second delemiter.",
ErrUsynBool,		"Invalid syntax - ON or OFF must be specified.",
ErrUstart_syntax,	"Invalid syntax - @filename.",
ErrUstart_file,		"Can't open command file:",
ErrUnotconnected,	"Database not connected.",
ErrUdbnoname,		"Database name not specified.",
ErrUhostnamelong,	"Hostname too long.",
ErrUdbnamelong,		"Database name too long.",
ErrUnosuchbuf,		"No such buffer.",
ErrUscrsize,		"Screen size too small.",
ErrUnumbufbig,		"Number of buffers too big.",
ErrUnumbufsmall,	"Number of buffers too small.",
ErrUnosuchflag,		"No such flag. Allowed: CREATE,REPLACE,APPEND",
ErrUnofilename,		"No file name specified.",
ErrUnospool,		"Currently spooling turned off.",
ErrUmarginvalid,	"Size of margin should not be less than 0.",
ErrUmarginbig,		"Size of margin should not be grater than ",
ErrUendbarno,		"Not possible print endbar. Check size of margins.",
ErrUheadingno,		"Not possible print heading. Check size of margins.",
ErrUnohelp,		"No help available.",
ErrUstringlong,		"String too long.",
ErrUshellexec,		"Can't execute a command.",
ErrUsynwrap,		"Invalid wrap type.",
ErrUpagebreakinv,	"Invalid page break mode. Must be ON, OFF or a number 1-255.",
/* INTERNAL errors ********************************************/

ErrIalloc,		"Can't allocate memory.",
ErrIargc,		"Invalid number of arguments.",
ErrIfile_i_open,	"Can't open input file -",
ErrItmpfile,		"Can't open tmp file -",
ErrIfilewrite,		"Write on file failed -",
ErrIcantedit,		"Can't exec editor -",
ErrIsave_open,		"Save failed. Can't open file -",
ErrIspoolwopen,		"Can't open spool file for writing -",
ErrIfopen,		"Can't open file -",

/* SERVER errors **********************************************/

ErrServer,		"",
ErrSconnect,		"Connect -",
ErrSseldb,		"Select DB -",

0,NULL_char };

void
Err(code,add)
int	code;
char	*add;
{
	register struct err_s	*ep;

	oErr("\nERROR (");

	if(code < 100)
		oErr("User");
	else if(code < 200)
		oErr("Internal");
	else
		oErr("Server");

	oErr("): ");

	ep = Errors;

	while(ep->code > 0){
		if(ep->code == code)
			break;
		ep++;
	}
	if(ep->code == 0)
		oErr("Unknown error.");
	else
		oErr(ep->msg);

	if(add != NULL_char){
		oErr(" ");
		if(strcmp(add,"E") == 0)
			oErr(strerror(errno));
		else
			oErr(add);
	}

	oErr("\n\n");
}

#ifdef NO_STRERROR
char *
strerror(num)
int num;
{
	extern int sys_nerr;
	extern char *sys_errlist[];

	if((unsigned int)num > sys_nerr)
		return("Unknown error code");

	return(sys_errlist[num]);
}

#endif
